package org.one.stone.soup.file;

import java.io.*;
import java.util.*;

import org.one.stone.soup.util.*;

/*
 * Wet-Wired.com Library Version 2.1
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

 /**
	* The FilesWatch class calls the fileChanged method of registered
	* listeners implementing the FileChangeListener when any of a
	* number of chosen files modified time stamp changes.
	*
	* Instances of FileChangeListener can register to be informed when
	* a file changes by calling addListener.
	*
*/

public class FilesWatch implements java.awt.event.ActionListener{

	private boolean processing=false;
	private TimeWatch timer;
	private File[] file = new File[0];
	private long[] ts = new long[0];
	private Vector listeners = new Vector();
	/**
	 * Creates an instance of FilesWatch to monitor changes to a list of files
	 * @param title			the title of the timing thread
	 * @param refreshTime	the time between rechecking the file last modified time in 1/1000 seconds
	 */

public FilesWatch(String title,int refreshTime) {

	timer = new TimeWatch(refreshTime,title);
	timer.addActionListener(this);
	timer.setLooped(true);
	timer.start();
}
/**
 * called by the timer to recheck file modified times.
 */
public void actionPerformed(java.awt.event.ActionEvent e)
{
	if(processing)
	{
		return;
	}
	processing = true;
	for(int loop=0;loop<file.length;loop++)
	{
		if(file[loop].isDirectory())
		{
			long time = calculateFolderLastModified(file[loop]);
			if(time!=ts[loop])
			{
				notifyListeners(file[loop]);
				ts[loop] = time;
			}			
		}
		else
		{
			if(file[loop].lastModified()!=ts[loop])
			{
				notifyListeners(file[loop]);
				ts[loop] = file[loop].lastModified();
			}
		}
	}
	processing=false;
}
	/**
	 * Adds the specified file to the list of files being monitored.
	 *
	 * @param    newFile	the new file to watch.
	 */

public void addFolder(File newFolder)
{
	if(!newFolder.exists() || !newFolder.isDirectory())
	{
		return;
	}

	File[] fileB = new File[file.length+1];
	long[] tsB = new long[ts.length+1];

	System.arraycopy(file,0,fileB,0,file.length);
	System.arraycopy(ts,0,tsB,0,ts.length);

	fileB[file.length] = newFolder;
	tsB[ts.length] = calculateFolderLastModified(newFolder);

	ts=tsB;
	file = fileB;
}

private long calculateFolderLastModified(File newFolder)
{
	return calculateFolderLastModified(newFolder,0);
}
private long calculateFolderLastModified(File newFolder,long start)
{
	start+=newFolder.lastModified();
	
	File[] list = newFolder.listFiles();

	for(int loop=0;loop<list.length;loop++)
	{
		if(list[loop].isDirectory())
		{
			start = calculateFolderLastModified(list[loop],start);
		}
		else
		{
			start+=list[loop].lastModified();
		}
	}
	
	return start;
}

public void addFile(File newFile)
{
	if(!newFile.exists() || newFile.isDirectory())
	{
		return;
	}

	File[] fileB = new File[file.length+1];
	long[] tsB = new long[ts.length+1];

	System.arraycopy(file,0,fileB,0,file.length);
	System.arraycopy(ts,0,tsB,0,ts.length);

	fileB[file.length] = newFile;
	tsB[ts.length] = newFile.lastModified();

	ts=tsB;
	file = fileB;
}
	/**
	 * Adds the specified file change listener to receive file change
	 * events from this component.
	 *
	 * @param    listener	the file change listener.
	 */

public void addListener(FileChangeListener listener)
{
	listeners.addElement(listener);
}
/**
 * Notifies the listeners that a file has changed
 */
private void notifyListeners(File changedFile)
{
	for(int loop=0;loop<listeners.size();loop++)
	{
		FileChangeListener listener = (FileChangeListener)listeners.elementAt(loop);
		listener.fileChanged(changedFile);
	}
}
	/**
	 * Removes the specified file change listener from the list of
	 * registered listeners.
	 *
	 * @param    listener	the file change listener to remove.
	 */

public void removerListener(FileChangeListener listener)
{
	listeners.removeElement(listener);
}

public void stop()
{
	timer.stop();
}
}
