package org.one.stone.soup.mjdb.data.field;

import org.one.stone.soup.stringhelper.*;
import java.util.*;
import org.one.stone.soup.mjdb.data.functions.*;


/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* The Field class is the superclass of all the mjdb data wrappers.
	* The class controls a set of functionality for all sub classes:
	* Field locking.
	* Change monitor.
	* Change listeners.
	* DataLinker registration.
	* Unique reference.
	* @author Nik Cross
*/

public abstract class Field implements Lockable{

	public static final int INT_MAX_NEG = Integer.MIN_VALUE;
	public static final int INT_MAX_POS = Integer.MAX_VALUE;

	public static final long LONG_MAX_NEG = Long.MIN_VALUE;
	public static final long LONG_MAX_POS = Long.MAX_VALUE;

	public static final int DEGREE_MIN=0;
	public static final int DEGREE_MAX=360;

	public static final int COLOR_MIN=0;
	public static final int COLOR_MAX=255;

	public static final float FLOAT_MIN_POS = Float.MIN_VALUE;
	public static final float FLOAT_MIN_NEG = -FLOAT_MIN_POS;
	public static final float FLOAT_MAX_POS = Float.MAX_VALUE;
	public static final float FLOAT_MAX_NEG = -FLOAT_MAX_POS;

	private String ID = "_Object";
	private String ref = StringGenerator.generateUniqueId();

	public boolean isReference = false;
	private boolean changed = true;

	public static final int GT = 1;
	public static final int LT = -1;
	public static final int EQ = 0;

	private Vector listeners = new Vector();
	protected boolean report = false;
	protected boolean locked = false;
	private LockOwner lockOwner = null;
public void addListener(FieldListener listener) {
	listeners.addElement(listener);
	report=true;
}
public int compare(Object target) throws CompareException{

	try{
		Field field = (Field)Wrapper.wrap(target);
		return compare(field);
	}
	catch(Exception e)
	{
		throw new CompareException("Can't compare non fields:"+e+"("+Wrapper.wrap(target)+")");
	}
}
public abstract int compare(Field target) throws CompareException;
public void forceLock(LockOwner owner) {

	locked = true;
	this.lockOwner = owner;
}
public void forceUnlock(LockOwner owner) {

	lockOwner = null;
	locked = false;
}
public Vector getClonedElements() {
	return new Vector();
}
public Vector getClonedElements(String path) {
	return new Vector();
}
/**
 * Insert the method's description here.
 * Creation date: (23/07/00 08:49:37)
 * @return java.lang.String
 */
public String getId() {
	return ID;
}
public LockOwner getLockOwner() {
	return lockOwner;
}
/**
 * Insert the method's description here.
 * Creation date: (23/07/00 08:49:37)
 * @return java.lang.String
 */
public String getRef() {
	return ref;
}
public abstract String getValue(String name);
public boolean hasChanged() {

//	if(id.charAt(0)=='#')
//		System.out.println("check:"+id+ref+"="+this);

	return changed;
}
public boolean inRange(Object min,Object max) throws CompareException
{
	return inRange(min,true,max,true);

}
public boolean inRange(Object min,boolean inclusiveMin,Object max,boolean inclusiveMax) throws CompareException
{
	int minState=compare(Wrapper.wrap(min));
	int maxState=compare(Wrapper.wrap(max));
	if(
		(minState==LT || (inclusiveMin && minState==EQ)) &&
		(maxState==GT || (inclusiveMax && maxState==EQ))
	)
		return true;
	else
		return false;

}
public boolean inRange(Field min,Field max) throws CompareException
{
	return inRange(min,true,max,true);

}
public boolean inRange(Field min,boolean inclusiveMin,Field max,boolean inclusiveMax) throws CompareException
{
	int minState=compare(min);
	int maxState=compare(max);

	if(
		(minState==GT || (inclusiveMin && minState==EQ)) &&
		(maxState==LT || (inclusiveMax && maxState==EQ))
	)
		return true;
	else
		return false;

}
public boolean isLocked() {
	return locked;
}
public void lock(LockOwner owner) throws LockException {
	if(locked)
	{
		throw new LockException(""+this+" is already locked by "+lockOwner);
	}

	locked = true;
	lockOwner = owner;
}
public void register(DataLinker linker) {}
public void removeAllListeners() {
	listeners = new Vector();
	report = false;
}
public void removeListener(FieldListener listener) {
	listeners.removeElement(listener);
	if(listeners.size()==0)
		report=false;
}
public void replace(Field oldObj, Field newObj){}
public void reportChange(Object source) {
	int loop;
	FieldListener currentListener;
	int size = listeners.size();

	if(source instanceof FieldListener)
	{
		for(loop=0;loop<size;loop++)
		{
			currentListener=((FieldListener)listeners.elementAt(loop));
			if(source!=currentListener)
				currentListener.receiveChange(source,this);
		}
	}
	else
	{
		for(loop=0;loop<size;loop++)
		{
			currentListener=((FieldListener)listeners.elementAt(loop));
			currentListener.receiveChange(source,this);
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (23/07/00 08:45:24)
 * @param newChanged boolean
 */
public void resetChanged() {
	changed = false;
}
/**
 * Insert the method's description here.
 * Creation date: (23/07/00 08:45:24)
 * @param newChanged boolean
 */
public void setChanged() {
	changed = true;

/*	if(org.one.stone.soup.igml.IGMLEngine.bmChanges!=null)
	{
		org.one.stone.soup.igml.IGMLEngine.bmChanges.inc();
		if(org.one.stone.soup.igml.IGMLEngine.bmChanges.fullReport)
			org.one.stone.soup.igml.IGMLEngine.bmChanges.messages.addElement(this.getId());
	}*/
//	if(id.charAt(0)=='#')
//		System.out.println("set:"+id+ref+"="+this);
}
public void setId(String newId,Object source) throws LockException {
	if(locked)
		throw new LockException(this+"is locked");

	ID = newId;
}
public void unlock(LockOwner owner) throws LockException{
	if(locked==false)
		return;

	if(owner==lockOwner)
	{
		lockOwner=null;
		locked=false;
	}
	else
		throw new LockException(""+this+" is locked by "+lockOwner);
}

public boolean isLockOwner(Object testOwner) {
	if(lockOwner==testOwner)
	{
		return true;
	}
	else
	{
		return false;
	}
}
}
