package org.one.stone.soup.mjdb.data.field;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* The IntField class is a wrapper for a single int value.
	* @author Nik Cross
*/

public class IntField extends Field {
	public static String DEFAULT_ID = "_Int";

	private int valueMax = INT_MAX_POS;
	private int valueMin = INT_MAX_NEG;

	private int data = 0;

	private boolean circularRange=false;
/**
 * This constructor creates a new IntField with a value of 0 and
 * the circularRange flag set to false.
 */

public IntField() {
	super();
}
/**
 * This constructor sets the initial int value.
 * The circularRange flag is set to false.
 * @param data	The int value
 */

public IntField(int data) {
	super();

	this.data=data;
}
/**
 * This constructor sets the initial int value, the minimum value and
 * the maximum value.
 * The circularRange flag is set to false.
 * @param data	The int value
 * @param min	The minimum value
 * @param min	The maximum value
 */

public IntField(int data,int min,int max) {
	super();

	this.data=data;
	valueMin=min;
	valueMax=max;
}
/**
 * This constructor sets the initial int value, the minimum value,
 * the maximum value and the circularRange falg value.
 * If the circularRange flag is set to true,
 *  when the int value is set below the minimum value it changes
 *   to the maximum.
 *  when the int value is set above the maximum value it changes
 *   to the minimum.
 * @param data	The int value
 * @param min	The minimum value
 * @param min	The maximum value
 * @param circular	The circularRange flag value.
 */

public IntField(int data,int min,int max,boolean circular) {
	super();

	this.data=data;
	this.circularRange=circular;
	valueMin=min;
	valueMax=max;
}
private int capValue(int v) {

	if(isCircularRange())
	{
		while(v<valueMin)
			v+=valueMax;
		while(v>valueMax)
			v-=valueMax;
	}
	else
	{
		if(v<valueMin)
			v=valueMin;

		if(v>valueMax)
			v=valueMax;
	}

	return v;
}
/**
 * Compares this object with the target object.
 * Returns EQ if the two int values are the same.
 * Returns GT if the the target int value is greater than the target int value.
 * Returns LT if the the target int value is less than the target int value.
 * @param target	The Field to compare with (Accepts FloatField,IntField)
 */

public int compare(Field target) throws CompareException {

	if(target instanceof IntField)
	{
		if( ((IntField)target).getValue() == this.data)
			return EQ;
		else if( ((IntField)target).getValue() > this.data)
			return GT;
		else
			return LT;
	}
	else if(target instanceof FloatField)
	{
		if( ((FloatField)target).getValue() == this.data)
			return EQ;
		else if( ((FloatField)target).getValue() > this.data)
			return GT;
		else
			return LT;
	}

	throw new CompareException("Can't compare "+target+" with int");

}
/**
 * Decrement the int value by one.
 * @param source	The object requesting the value change
 */
public void dec(Object source) throws LockException {

	if(locked)
		throw new LockException(this+"is locked");

	data --;
	setChanged();

	if(report)
		reportChange(source);
}
/**
 * Returns the int data held in this field.
 */

public int getValue() {
	return data;
}
/**
 * Returns a field held in this field as a String.
 * If name = "value" this method returns the int value
 * If name = "minValue" this method returns the minimum int value
 * If name = "maxValue" this method returns the maximum int value
 * @param name	The name of the field to return
 */
public String getValue(String name) {

	if(name.equals("value"))
	{
		return ""+data;
	}
	else if(name.equals("minValue"))
	{
		return ""+valueMin;
	}
	else if(name.equals("maxValue"))
	{
		return ""+valueMax;
	}
	else return "?";
}
/**
 * Returns the maximum int value held in this field.
 */

public int getValueMax() {
	return valueMax;
}
/**
 * Returns the minimum int value held in this field.
 */

public int getValueMin() {
	return valueMin;
}
/**
 * Increment the int value by one.
 * @param source	The object requesting the value change
 */
public void inc(Object source) throws LockException {

	if(locked)
		throw new LockException(this+"is locked");

	data ++;
	setChanged();

	if(report)
		reportChange(source);
}
private boolean isCircularRange() {
	return circularRange;
}
/**
 * Set the value of the circularRange flag.
 * If the circularRange flag is set to true,
 *  when the int value is set below the minimum value it changes
 *   to the maximum.
 *  when the int value is set above the maximum value it changes
 *   to the minimum.
 * @param newCircularRange	The flag state
 */

public void setCircularRange(boolean newCircularRange) {
	circularRange = newCircularRange;
}
/**
 * Set the minimum and maximum values for this int value.
 * @param newValueMin	The minimum int value
 * @param newValueMax	The maximum int value
 */

public void setRange(int newValueMin,int newValueMax) {

	this.valueMin = newValueMin;
	this.valueMax = newValueMax;
}
/**
 * Set the int value.
 * @param newData		The new int value
 * @param source		The object requesting the value change
 */
public void setValue(int newData,Object source) throws LockException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	newData = capValue(newData);

	if(newData == data)
		return;

	data = newData;
	setChanged();

	if(report)
		reportChange(source);
}
/**
 * Set the int value.
 * If name = "value" then the int value is set
 * If name = "minValue" then the minimum int value is set
 * If name = "maxValue" then the maximum int value is set
 * @param name		The target field name
 * @param value		A String containing the new int value
 * @param source	The object requesting the value change
 */

public void setValue(String name, String value, Object source) throws LockException
{
	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	if(name.equals("value"))
	{
		setValue( Integer.parseInt(value),this );
	}
	else if(name.equals("minValue"))
	{
		setValueMin( Integer.parseInt(value) );
	}
	else if(name.equals("maxValue"))
	{
		setValueMax( Integer.parseInt(value) );
	}
}
/**
 * Set the maximum int value.
 * @param newData		The new maximum int value
 * @param source		The object requesting the value change
 */
public void setValueMax(int newValueMax) {
	valueMax = newValueMax;
}
/**
 * Set the minimum int value.
 * @param newData		The new minimum int value
 * @param source		The object requesting the value change
 */
public void setValueMin(int newValueMin) {
	valueMin = newValueMin;
}
public String toString() {
	return Integer.toString(data);
}
}
